/* OdLib_Io - Copyright (c) 2008-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.opendocument.transform;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import net.mapeadores.opendocument.io.ByteArrayOdSource;
import net.mapeadores.opendocument.io.CharSequenceOdSource;
import net.mapeadores.opendocument.io.FileOdSource;
import net.mapeadores.opendocument.io.OdUtils;
import net.mapeadores.opendocument.io.OdZip;
import net.mapeadores.opendocument.io.OdZipEngine;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.io.StreamTransformer;
import net.mapeadores.util.io.StreamTransformerErrorHandler;
import net.mapeadores.util.io.TransformerErrorListener;


/**
 *
 * @author Vincent Calame
 */
public class ODStreamTransformer implements StreamTransformer {

    private final static String HREF_ATTRIBUTE = "xlink:href=";
    private final static String BDF_PICTURE_PREFIX = "picture:";
    private final short odType;
    private final String extension;
    private final String mimeType;
    private final InternalURIResolver internalURIResolver = new InternalURIResolver();
    private final TransformerErrorListener internalErrorListener = new TransformerErrorListener();
    private Templates contentTemplate;
    private byte[] stylesByteArray;
    private byte[] contentAutomaticStylesByteArray;
    private File stylesFile;

    private ODStreamTransformer(String extension) {
        this.extension = extension;
        odType = OdUtils.getOdType(extension);
        mimeType = OdUtils.getMimeType(odType);
    }

    @Override
    public String getMimeType() {
        return mimeType;
    }

    @Override
    public String getExtension() {
        return extension;
    }

    @Override
    public void transform(Source source, OutputStream outputStream, Map<String, String> transformerParameters) throws TransformerException {
        OdZip odZip = OdZip.newInstance(odType);
        if (stylesByteArray != null) {
            odZip.setStylesOdSource(new ByteArrayOdSource(stylesByteArray));
        } else if (stylesFile != null) {
            odZip.setStylesOdSource(new FileOdSource(stylesFile));
        }
        Transformer transformer = contentTemplate.newTransformer();
        transformer.setURIResolver(internalURIResolver);
        transformer.setErrorListener(internalErrorListener);
        for (Map.Entry<String, String> entry : transformerParameters.entrySet()) {
            transformer.setParameter(entry.getKey(), entry.getValue());
        }
        StringWriter writer = new StringWriter();
        transformer.transform(source, new StreamResult(writer));
        StringBuffer buf = writer.getBuffer();
        //BdfPictureProviderBuilder providerBuilder = new BdfPictureProviderBuilder();
        check(buf);
        odZip.setContentOdSource(new CharSequenceOdSource(buf, "UTF-8"));
        try {
            OdZipEngine.run(outputStream, odZip);
        } catch (IOException ioe) {
            throw new TransformerException(ioe);
        } catch (NestedTransformerException nte) {
            throw nte.getTransformerException();
        }
    }

    private void check(StringBuffer buf) {
        /*BdfPictureProviderBuilder providerBuilder = new BdfPictureProviderBuilder();
         int idx = buf.indexOf(HREF_ATTRIBUTE);
         int idx_debut;
         int idx_fin;

         PictureConverter converter = new PictureConverter();
         while (idx != -1) {
         idx_debut = idx + HREF_ATTRIBUTE.length() + 1;
         String quoteChar = buf.substring(idx_debut - 1, idx_debut);
         idx_fin = buf.indexOf(quoteChar, idx_debut);
         String path = buf.substring(idx_debut, idx_fin);
         if (path.startsWith(BDF_PICTURE_PREFIX)) {
         path = path.substring(BDF_PICTURE_PREFIX.length());
         String pictureName = converter.getName(path);
         buf.delete(idx_debut, idx_fin);
         buf.insert(idx_debut, pictureName);
         idx_fin = idx_debut + pictureName.length();
         }
         idx = buf.indexOf(HREF_ATTRIBUTE, idx_fin);
         }*/
    }


    public static class Builder {

        private final ODStreamTransformer odStreamTransformer;
        private final TransformerErrorListener factoryErrorListener = new TransformerErrorListener();
        private StreamTransformerErrorHandler streamTransformerErrorHandler;
        private String contentURI;

        public Builder(String extension) {
            odStreamTransformer = new ODStreamTransformer(extension);
        }

        public ODStreamTransformer toODStreamTransformer() {
            return odStreamTransformer;
        }

        public void setStyles(InputStream inputStream) throws IOException {
            setStyles(IOUtils.toByteArray(inputStream));
        }

        public void setStyles(byte[] byteArray) {
            odStreamTransformer.stylesByteArray = byteArray;
            odStreamTransformer.stylesFile = null;
        }

        public void setStyles(File stylesFile) {
            odStreamTransformer.stylesByteArray = null;
            odStreamTransformer.stylesFile = stylesFile;
        }

        public void setContentAutomaticStyles(InputStream inputStream) throws IOException {
            setContentAutomaticStyles(IOUtils.toByteArray(inputStream));
        }

        public void setContentAutomaticStyles(byte[] byteArray) {
            odStreamTransformer.contentAutomaticStylesByteArray = byteArray;
        }

        public void setContentXsl(byte[] byteArray) {
            StreamSource streamSource = new StreamSource(new ByteArrayInputStream(byteArray));
            streamSource.setSystemId(contentURI);
            initTemplates(streamSource);
        }

        public void setContentXsl(String s) {
            StreamSource streamSource = new StreamSource(new StringReader(s));
            streamSource.setSystemId(contentURI);
            initTemplates(streamSource);
        }

        public void setContentXsl(InputStream inputStream) {
            StreamSource streamSource = new StreamSource(inputStream);
            streamSource.setSystemId(contentURI);
            initTemplates(streamSource);
        }

        public void setURIResolver(URIResolver uriResolver) {
            odStreamTransformer.internalURIResolver.setURIResolver(uriResolver);
        }

        public void setErrorListener(ErrorListener errorListener) {
            factoryErrorListener.setCustomErrorListener(errorListener);
            odStreamTransformer.internalErrorListener.setCustomErrorListener(errorListener);
        }

        private void initTemplates(Source source) {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            tFactory.setURIResolver(odStreamTransformer.internalURIResolver);
            tFactory.setErrorListener(factoryErrorListener);
            try {
                odStreamTransformer.contentTemplate = tFactory.newTemplates(source);
            } catch (TransformerException te) {
                factoryErrorListener.checkFatalTransformerException(te);
            }
            if (factoryErrorListener.hasError()) {
                streamTransformerErrorHandler.wrongResourceError(contentURI, "XSL error :: " + factoryErrorListener.getErrorMessage());
            }
        }

        public void setStreamTransformerErrorHandler(StreamTransformerErrorHandler streamTransformerErrorHandler) {
            this.streamTransformerErrorHandler = streamTransformerErrorHandler;
        }

        public void setContentURI(String contentURI) {
            this.contentURI = contentURI;
        }

    }

    public byte[] getStylesByteArray() {
        return stylesByteArray;
    }

    public byte[] getContentAutomaticStylesByteArray() {
        return contentAutomaticStylesByteArray;
    }

    public File getStylesFile() {
        return stylesFile;
    }


    private class InternalURIResolver implements URIResolver {

        private URIResolver customURIResolver;

        private void setURIResolver(URIResolver customURIResolver) {
            this.customURIResolver = customURIResolver;
        }

        @Override
        public Source resolve(String href, String base) throws TransformerException {
            if (href.equals("_content_automaticstyles.xml")) {
                byte[] byteArray = getContentAutomaticStylesByteArray();
                if (byteArray != null) {
                    return new StreamSource(new ByteArrayInputStream(byteArray));
                }
            } else if (href.equals("styles.xml")) {
                byte[] byteArray = getStylesByteArray();
                if (byteArray != null) {
                    return new StreamSource(new ByteArrayInputStream(byteArray));
                }
                File stylesFile = getStylesFile();
                if (stylesFile != null) {
                    return new StreamSource(stylesFile);
                }
            }
            if (customURIResolver != null) {
                return customURIResolver.resolve(href, base);
            } else {
                return null;
            }
        }

    }


    private static class PictureConverter {

        private final Map<String, String> nameMap = new HashMap<String, String>();
        private int current = 1;


        private PictureConverter() {

        }

        private String getName(String path) {
            String name = nameMap.get(path);
            if (name == null) {
                int idx = path.lastIndexOf(".");
                int idx2 = path.lastIndexOf("/");
                if (idx2 > idx) {
                    idx = -1;
                }
                if (idx != -1) {
                    name = "bdf-" + current + "." + path.substring(idx);
                } else {
                    name = "bdf-" + current;
                }
                current++;
                nameMap.put(path, name);
            }
            return name;
        }

    }

}
