/* ScrutariLib_DB - Copyright (c) 2010-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.codes;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.RandomAccess;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.db.api.codes.ElementCodes;
import net.scrutari.db.api.codes.IndexationCodes;


/**
 *
 * @author Vincent Calame
 */
public class CodesUtils {

    public final static Codes EMPTY_CODES = new EmptyCodes();
    public final static ElementCodes EMPTY_ELEMENTCODES = new EmptyElementCodes();
    public final static IndexationCodes EMPTY_INDEXATIONCODES = new EmptyIndexationCodes();

    public static Codes toCodes(int[] intArray) {
        if (intArray == null) {
            return EMPTY_CODES;
        }
        if (intArray.length == 0) {
            return EMPTY_CODES;
        }
        return new ArrayCodes(intArray);
    }

    public static Codes toCodes(Collection<Integer> codeCollection) {
        if (codeCollection == null) {
            return EMPTY_CODES;
        }
        int size = codeCollection.size();
        if (size == 0) {
            return EMPTY_CODES;
        }
        if (size == 1) {
            return new UniqueCodes(codeCollection.iterator().next());
        }
        int[] result = new int[size];
        int p = 0;
        for (Integer itg : codeCollection) {
            result[p] = itg;
            p++;
        }
        return new ArrayCodes(result);
    }

    public static ElementCodes toElementCodes(List<Integer> elementCodeList, List<Integer> subsetCodeList) {
        if (elementCodeList == null) {
            return EMPTY_ELEMENTCODES;
        }
        int size = elementCodeList.size();
        if (size == 0) {
            return EMPTY_ELEMENTCODES;
        }
        if (size == 1) {
            return new UniqueElementCodes(elementCodeList.get(0), subsetCodeList.get(0));
        }
        int[] result = new int[size];
        int[] subsetResult = new int[size];
        for (int i = 0; i < size; i++) {
            result[i] = elementCodeList.get(i);
            subsetResult[i] = subsetCodeList.get(i);
        }
        return new ArrayElementCodes(result, subsetResult);
    }

    public static ElementCodes toElementCodes(Map<Integer, Integer> elementCodeListMap) {
        if (elementCodeListMap == null) {
            return EMPTY_ELEMENTCODES;
        }
        int size = elementCodeListMap.size();
        if (size == 0) {
            return EMPTY_ELEMENTCODES;
        }
        int[] result = new int[size];
        int[] subsetResult = new int[size];
        int p = 0;
        for (Map.Entry<Integer, Integer> entry : elementCodeListMap.entrySet()) {
            result[p] = entry.getKey();
            subsetResult[p] = entry.getValue();
            p++;
        }
        return new ArrayElementCodes(result, subsetResult);
    }

    public static IndexationCodes toIndexationCodes(List<Integer> elementCodeList, List<Float> ponderationList) {
        if (elementCodeList == null) {
            return EMPTY_INDEXATIONCODES;
        }
        int size = elementCodeList.size();
        if (size == 0) {
            return EMPTY_INDEXATIONCODES;
        }
        if (size == 1) {
            return new UniqueIndexationCodes(elementCodeList.get(0), ponderationList.get(0));
        }
        int[] result = new int[size];
        float[] ponderationResult = new float[size];
        for (int i = 0; i < size; i++) {
            result[i] = elementCodeList.get(i);
            ponderationResult[i] = ponderationList.get(i);
        }
        return new ArrayIndexationCodes(result, ponderationResult);
    }

    public static Codes shuffle(Codes codes, int finalSize) {
        int count = codes.getCodeCount();
        if (count < 2) {
            return codes;
        }
        List<Integer> list = new ArrayList<Integer>();
        for (int i = 0; i < count; i++) {
            list.add(codes.getCode(i));
        }
        Collections.shuffle(list);
        if ((finalSize == -1) || (count <= finalSize)) {
            return toCodes(list);
        } else {
            int[] array = new int[finalSize];
            for (int i = 0; i < finalSize; i++) {
                array[i] = list.get(i);
            }
            return new ArrayCodes(array);
        }
    }

    public static List<BaseURI> wrap(BaseURI[] array) {
        return new BaseURIList(array);
    }

    public static List<CorpusURI> wrap(CorpusURI[] array) {
        return new CorpusURIList(array);
    }

    public static List<ScrutariDataURI> wrap(ScrutariDataURI[] array) {
        return new ScrutariDataURIList(array);
    }


    private static class EmptyCodes implements Codes {

        private EmptyCodes() {
        }

        @Override
        public int getCodeCount() {
            return 0;
        }

        @Override
        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyElementCodes implements ElementCodes {

        private EmptyElementCodes() {
        }

        @Override
        public int getCodeCount() {
            return 0;
        }

        @Override
        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public int getSubsetCode(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyIndexationCodes implements IndexationCodes {

        private EmptyIndexationCodes() {
        }

        @Override
        public int getCodeCount() {
            return 0;
        }

        @Override
        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public float getPonderation(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayCodes implements Codes {

        private final int[] array;

        private ArrayCodes(int[] array) {
            this.array = array;
        }

        @Override
        public int getCodeCount() {
            return array.length;
        }

        @Override
        public int getCode(int i) {
            return array[i];
        }

    }


    private static class ArrayElementCodes extends ArrayCodes implements ElementCodes {

        private final int[] subsetCodeArray;

        private ArrayElementCodes(int[] array, int[] subsetCodeArray) {
            super(array);
            this.subsetCodeArray = subsetCodeArray;
        }

        @Override
        public int getSubsetCode(int i) {
            return subsetCodeArray[i];
        }

    }


    private static class ArrayIndexationCodes extends ArrayCodes implements IndexationCodes {

        private final float[] ponderationArray;

        private ArrayIndexationCodes(int[] array, float[] ponderationArray) {
            super(array);
            this.ponderationArray = ponderationArray;
        }

        @Override
        public float getPonderation(int i) {
            return ponderationArray[i];
        }

    }


    private static class UniqueCodes implements Codes {

        private final int unique;

        private UniqueCodes(int unique) {
            this.unique = unique;
        }

        @Override
        public int getCodeCount() {
            return 1;
        }

        @Override
        public int getCode(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return unique;
        }

    }


    private static class UniqueElementCodes extends UniqueCodes implements ElementCodes {

        private final int subsetCode;

        private UniqueElementCodes(int unique, int subsetCode) {
            super(unique);
            this.subsetCode = subsetCode;
        }

        @Override
        public int getSubsetCode(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return subsetCode;
        }

    }


    private static class UniqueIndexationCodes extends UniqueCodes implements IndexationCodes {

        private final float ponderation;

        private UniqueIndexationCodes(int unique, float ponderation) {
            super(unique);
            this.ponderation = ponderation;
        }

        @Override
        public float getPonderation(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return ponderation;
        }

    }


    private static class BaseURIList extends AbstractList<BaseURI> implements RandomAccess {

        private final BaseURI[] array;

        private BaseURIList(BaseURI[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public BaseURI get(int index) {
            return array[index];
        }

    }


    private static class ScrutariDataURIList extends AbstractList<ScrutariDataURI> implements RandomAccess {

        private final ScrutariDataURI[] array;

        private ScrutariDataURIList(ScrutariDataURI[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public ScrutariDataURI get(int index) {
            return array[index];
        }

    }


    private static class CorpusURIList extends AbstractList<CorpusURI> implements RandomAccess {

        private final CorpusURI[] array;

        private CorpusURIList(CorpusURI[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public CorpusURI get(int index) {
            return array[index];
        }

    }

}
