/* ScrutariLib_SearchEngine - Copyright (c) 2013-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.scrutari.data.FicheData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.operands.eligibility.CodeEligibility;
import net.scrutari.searchengine.api.operands.eligibility.Eligibility;
import net.scrutari.searchengine.api.operands.eligibility.FicheDataEligibility;
import net.scrutari.searchengine.api.operands.eligibility.SubEligibility;


/**
 *
 * @author Vincent Calame
 */
public class EligibilityBuffer {

    private final DataAccess dataAccess;
    private final Map<Integer, Boolean> bufferMap = new HashMap<Integer, Boolean>();
    private final Eligibility ficheEligibilityOperand;
    private Set<Lang> langCheckSet;
    private FicheData currentFicheData;
    private boolean currentFicheDataInit;

    public EligibilityBuffer(DataAccess dataAccess, Eligibility ficheEligibility) {
        this.dataAccess = dataAccess;
        this.ficheEligibilityOperand = ficheEligibility;
    }

    public void addLangCheck(Lang[] langArray) {
        langCheckSet = new HashSet<Lang>();
        for (Lang lang : langArray) {
            langCheckSet.add(lang);
        }
    }

    public boolean acceptFiche(int codefiche) {
        if ((ficheEligibilityOperand == null) & (langCheckSet == null)) {
            return true;
        }
        Boolean bool = bufferMap.get(codefiche);
        if (bool != null) {
            return bool;
        }
        boolean bl = checkAcceptCodeFiche(codefiche);
        bufferMap.put(codefiche, bl);
        return bl;
    }

    public boolean acceptMotcle(int codemotcle) {
        return true;
    }

    private void clearCurrent() {
        currentFicheData = null;
        currentFicheDataInit = false;
    }

    private boolean checkAcceptCodeFiche(int code) {
        boolean test = true;
        if (ficheEligibilityOperand != null) {
            test = testEligibility(ficheEligibilityOperand, code);
        }
        if ((test) && (langCheckSet != null)) {
            Lang ficheLang;
            if (currentFicheData != null) {
                ficheLang = currentFicheData.getLang();
            } else {
                ficheLang = dataAccess.getFicheLang(code);
            }
            if (!langCheckSet.contains(ficheLang)) {
                test = false;
            }
        }
        clearCurrent();
        return test;
    }

    private boolean testEligibility(Eligibility eligibility, int code) {
        if (eligibility instanceof CodeEligibility) {
            return ((CodeEligibility) eligibility).acceptCode(code);
        } else if (eligibility instanceof FicheDataEligibility) {
            return ((FicheDataEligibility) eligibility).acceptFicheData(getFicheData(code));
        } else if (eligibility instanceof SubEligibility) {
            SubEligibility subEligibility = (SubEligibility) eligibility;
            short operator = subEligibility.getOperator();
            if (operator == LogicalOperationConstants.UNION_OPERATOR) {
                return testUnion(subEligibility, code);
            } else if (operator == LogicalOperationConstants.INTERSECTION_OPERATOR) {
                return testIntersection(subEligibility, code);
            } else {
                throw new SwitchException("Unknown operator = " + operator);
            }
        } else {
            throw new SwitchException("Unknown eligibility instance = " + eligibility.getClass().getName());
        }

    }

    private boolean testUnion(SubEligibility subEligibility, int code) {
        int count = subEligibility.getEligibilityCount();
        for (int i = 0; i < count; i++) {
            Eligibility eligibility = subEligibility.getEligibility(i);
            boolean test = testEligibility(eligibility, code);
            if (test) {
                return true;
            }
        }
        return false;
    }

    private boolean testIntersection(SubEligibility subEligibility, int code) {
        int count = subEligibility.getEligibilityCount();
        for (int i = 0; i < count; i++) {
            Eligibility eligibility = subEligibility.getEligibility(i);
            boolean test = testEligibility(eligibility, code);
            if (!test) {
                return false;
            }
        }
        return true;
    }

    private FicheData getFicheData(int code) {
        if (!currentFicheDataInit) {
            currentFicheDataInit = true;
            currentFicheData = dataAccess.getFicheDataByCode(code);
        }
        return currentFicheData;
    }

}
