/* ScrutariLib_Xml - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.SimpleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.datauri.BaseURI;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;


/**
 *
 * @author Vincent Calame
 */
public class BaseURIXmlEngine {

    private static final int AUTHORITY = 1;
    private static final int BASE_NAME = 2;
    private static final short START = 0;
    private static final short ON_BASE = 1;
    private static final short ON_METADATA = 2;
    private final XmlPullParserFactory factory;
    private final MessageHandler messageHandler;

    public BaseURIXmlEngine(MessageHandler messageHandler) {
        this.messageHandler = messageHandler;
        try {
            this.factory = XmlPullParserFactory.newInstance(System.getProperty(XmlPullParserFactory.PROPERTY_NAME), null);
        } catch (XmlPullParserException xppe) {
            throw new NestedLibraryException(xppe);
        }
    }

    public BaseURI read(InputStream inputStream) throws XmlPullParserException, IOException {
        XmlPullParser xpp = factory.newPullParser();
        xpp.setInput(inputStream, null);
        MetadataHandler metadataHandler = null;
        short state = START;
        int eventType = xpp.getEventType();
        do {
            if (eventType == XmlPullParser.START_TAG) {
                String tagName = xpp.getName();
                switch (state) {
                    case START:
                        if (tagName.equals("base")) {
                            state = ON_BASE;
                            break;
                        } else {
                            addSevere("_ error.wrong.xml.root", tagName, "base");
                            return null;
                        }
                    case ON_BASE:
                        if (tagName.equals("base-metadata")) {
                            state = ON_METADATA;
                            metadataHandler = new MetadataHandler();
                            break;
                        } else {
                            addSevere("_ error.unsupported.xml.missingtag", "/base", "base-metadata");
                            return null;
                        }
                    case ON_METADATA:
                        metadataHandler.processStartElement(tagName, null);
                        break;
                }

            } else if (eventType == XmlPullParser.END_TAG) {
                if (metadataHandler != null) {
                    boolean done = metadataHandler.processEndElement(xpp.getName());
                    if (done) {
                        return metadataHandler.getBaseURI();
                    }
                }
            } else if (eventType == XmlPullParser.TEXT) {
                if (metadataHandler != null) {
                    metadataHandler.processText(xpp.getText());
                }
            }
            eventType = xpp.next();
        } while (eventType != XmlPullParser.END_DOCUMENT);
        return null;
    }

    private void addSevere(String key, Object... values) {
        messageHandler.addMessage(ScrutariXmlConstants.SEVERE_CATEGORY, LocalisationUtils.toMessage(key, values));
    }


    private class MetadataHandler extends DefaultElementHandler {

        private String authority = null;
        private String baseName = null;

        private MetadataHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if ((tagname.equals("authority")) || (tagname.equals("authority-uuid"))) {
                return new SimpleElementHandler(AUTHORITY);
            }
            if (tagname.equals("base-name")) {
                return new SimpleElementHandler(BASE_NAME);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            if (text.length() == 0) {
                text = null;
            }
            switch (type) {
                case AUTHORITY:
                    authority = text;
                    break;
                case BASE_NAME:
                    baseName = text;
                    break;
            }
        }

        private BaseURI getBaseURI() {
            if (authority == null) {
                addSevere("_ error.unsupported.xml.missingtag", "/base/base-metadata", "authority");
            }
            if (baseName == null) {
                addSevere("_ error.unsupported.xml.missingtag", "/base/base-metadata", "base-name");
            }
            if ((authority != null) && (baseName != null)) {
                BaseURI baseURI = BaseURI.check(authority, baseName);
                if (baseURI == null) {
                    addSevere("_ error.wrong.xml.tagvalue", "/base/base-metadata", "authority|base-name");
                }
                return baseURI;
            } else {
                return null;
            }
        }

    }

}
