/* SctServer - Copyright (c) 2013-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.tools.ErrorUtils;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.AbstractList;
import java.util.Collections;
import java.util.List;
import java.util.RandomAccess;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.primitives.FuzzyDate;


/**
 *
 * @author Vincent Calame
 */
public final class CollectUtils {

    public final static List<ScrutariSource> EMPTY_SCRUTARISOURCELIST = Collections.emptyList();

    private CollectUtils() {
    }

    public static void initSslFactory() {
        try {
            SSLContext sc = SSLContext.getInstance("SSLv3");
            TrustManager[] tma = {new InternalTrustManager()};
            sc.init(null, tma, null);
            SSLSocketFactory ssf = sc.getSocketFactory();
            HttpsURLConnection.setDefaultSSLSocketFactory(ssf);
        } catch (NoSuchAlgorithmException | KeyManagementException e) {
            throw new NestedLibraryException(e);
        }
    }

    public static CollectResult toCollectResult(short state, FuzzyDate lastUpdate) {
        return new InternalCollectResult(state, lastUpdate);
    }

    public static boolean testConnection(URLConnection urlConnection, MessageHandler messageHandler) throws IOException {
        URL url = urlConnection.getURL();
        if (url.getProtocol().equals("https")) {
            HttpsURLConnection httpsCon = (HttpsURLConnection) urlConnection;
            int reponse = httpsCon.getResponseCode();
            if (reponse != HttpURLConnection.HTTP_OK) {
                ErrorUtils.severe(messageHandler, "_ error.exception.download", url.toString(), httpsCon.getResponseMessage());
                return false;
            } else {
                return true;
            }
        } else {
            HttpURLConnection con = (HttpURLConnection) urlConnection;
            int reponse = con.getResponseCode();
            if (reponse != HttpURLConnection.HTTP_OK) {
                ErrorUtils.severe(messageHandler, "_ error.exception.download", url.toString(), con.getResponseMessage());
                return false;
            } else {
                return true;
            }
        }
    }

    public static String stateToString(short state) {
        switch (state) {
            case CollectResult.OK_STATE:
                return "state_ok";
            case CollectResult.UNKNOWN_STATE:
                return "state_unknown";
            case CollectResult.INFO_NOCONNECTION_STATE:
                return "state_info_noconnection";
            case CollectResult.INFO_BROKENURL_STATE:
                return "state_info_brokenurl";
            case CollectResult.INFO_DOWNLOADERROR_STATE:
                return "state_info_downloaderror";
            case CollectResult.INFO_NOTXML_STATE:
                return "state_info_notxml";
            case CollectResult.INFO_WRONGXML_STATE:
                return "state_info_wrongxml";
            case CollectResult.DATA_NOCONNECTION_STATE:
                return "state_data_noconnection";
            case CollectResult.DATA_BROKENURL_STATE:
                return "state_data_brokenurl";
            case CollectResult.DATA_DOWNLOADERROR_STATE:
                return "state_data_downloaderror";
            case CollectResult.DATA_NOTXML_STATE:
                return "state_data_notxml";
            default:
                throw new SwitchException("state = " + state);
        }
    }

    public static String getStateMessageKey(short state) {
        switch (state) {
            case CollectResult.OK_STATE:
                return "_ state.source.ok";
            case CollectResult.UNKNOWN_STATE:
                return "_ state.source.unknown";
            case CollectResult.INFO_NOCONNECTION_STATE:
                return "_ state.source.info_noconnection";
            case CollectResult.INFO_BROKENURL_STATE:
                return "_ state.source.info_brokenurl";
            case CollectResult.INFO_DOWNLOADERROR_STATE:
                return "_ state.source.info_downloaderror";
            case CollectResult.INFO_NOTXML_STATE:
                return "_ state.source.info_notxml";
            case CollectResult.INFO_WRONGXML_STATE:
                return "_ state.source.info_wrongxml";
            case CollectResult.DATA_NOCONNECTION_STATE:
                return "_ state.source.data_noconnection";
            case CollectResult.DATA_BROKENURL_STATE:
                return "_ state.source.data_brokenurl";
            case CollectResult.DATA_DOWNLOADERROR_STATE:
                return "_ state.source.data_downloaderror";
            case CollectResult.DATA_NOTXML_STATE:
                return "_ state.source.data_notxml";
            default:
                throw new SwitchException("state = " + state);
        }
    }

    public static short stateToShort(String stateString) {
        if (stateString.equals("state_ok")) {
            return CollectResult.OK_STATE;
        } else if (stateString.equals("state_unknown")) {
            return CollectResult.UNKNOWN_STATE;
        } else if (stateString.equals("state_info_noconnection")) {
            return CollectResult.INFO_NOCONNECTION_STATE;
        } else if (stateString.equals("state_info_brokenurl")) {
            return CollectResult.INFO_BROKENURL_STATE;
        } else if (stateString.equals("state_info_downloaderror")) {
            return CollectResult.INFO_DOWNLOADERROR_STATE;
        } else if (stateString.equals("state_info_notxml")) {
            return CollectResult.INFO_NOTXML_STATE;
        } else if (stateString.equals("state_info_wrongxml")) {
            return CollectResult.INFO_WRONGXML_STATE;
        } else if (stateString.equals("state_data_noconnection")) {
            return CollectResult.DATA_NOCONNECTION_STATE;
        } else if (stateString.equals("state_data_brokenurl")) {
            return CollectResult.DATA_BROKENURL_STATE;
        } else if (stateString.equals("state_data_downloaderror")) {
            return CollectResult.DATA_DOWNLOADERROR_STATE;
        } else if (stateString.equals("state_data_notxml")) {
            return CollectResult.DATA_NOTXML_STATE;
        }
        throw new IllegalArgumentException("Unknown state = " + stateString);
    }

    public static List<ScrutariSource> wrap(ScrutariSource[] array) {
        return new ScrutariSourceList(array);
    }


    private static class InternalTrustManager implements X509TrustManager {

        private InternalTrustManager() {
        }

        @Override
        public void checkClientTrusted(X509Certificate[] chain, String authType) {
        }

        @Override
        public void checkServerTrusted(X509Certificate[] chain, String authType) {
        }

        @Override
        public X509Certificate[] getAcceptedIssuers() {
            return null;
        }

    }


    private static class InternalCollectResult implements CollectResult {

        private final short state;
        private final FuzzyDate lastUpdate;

        private InternalCollectResult(short state, FuzzyDate lastUpdate) {
            this.state = state;
            this.lastUpdate = lastUpdate;
        }

        @Override
        public short getState() {
            return state;
        }

        @Override
        public FuzzyDate getLastUpdate() {
            return lastUpdate;
        }

    }


    private static class ScrutariSourceList extends AbstractList<ScrutariSource> implements RandomAccess {

        private final ScrutariSource[] array;

        private ScrutariSourceList(ScrutariSource[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public ScrutariSource get(int index) {
            return array[index];
        }

    }

}
