/* BdfServer - Copyright (c) 2014 -2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.json;

import java.io.IOException;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.Langs;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageByLine;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.LabelUtils;


/**
 *
 * @author Vincent Calame
 */
public final class CommonJson {

    private CommonJson() {

    }

    public static void array(JSONWriter jsonWriter, Langs langList) throws IOException {
        int count = langList.getLangCount();
        jsonWriter.array();
        for (int i = 0; i < count; i++) {
            jsonWriter.value(langList.getLang(i).toString());
        }
        jsonWriter.endArray();
    }

    public static void object(JSONWriter jsonWriter, AttributeHolder attributeHolder) throws IOException {
        jsonWriter.object();
        properties(jsonWriter, attributeHolder);
        jsonWriter.endObject();
    }

    public static void properties(JSONWriter jsonWriter, AttributeHolder attributeHolder) throws IOException {
        int attributeCount = attributeHolder.getAttributeCount();
        for (int i = 0; i < attributeCount; i++) {
            Attribute attribute = attributeHolder.getAttributeByIndex(i);
            AttributeKey attributeKey = attribute.getAttributeKey();
            jsonWriter.key(attributeKey.toString());
            int valCount = attribute.getValueCount();
            jsonWriter.array();
            for (int j = 0; j < valCount; j++) {
                jsonWriter.value(attribute.getValue(j));
            }
            jsonWriter.endArray();
        }
    }

    public static void object(JSONWriter jsonWriter, LabelHolder labelHolder) throws IOException {
        jsonWriter.object();
        properties(jsonWriter, labelHolder);
        jsonWriter.endObject();
    }

    public static void properties(JSONWriter jsonWriter, LabelHolder labelHolder) throws IOException {
        int count = labelHolder.getLabelCount();
        for (int i = 0; i < count; i++) {
            Label label = labelHolder.getLabelByIndex(i);
            jsonWriter.key(label.getLang().toString());
            jsonWriter.value(label.getLabelString());
        }
    }

    public static void object(JSONWriter jsonWriter, CommandMessage commandMessage, MessageLocalisation messageLocalisation) throws IOException {
        jsonWriter.object();
        properties(jsonWriter, commandMessage, messageLocalisation);
        jsonWriter.endObject();
    }

    public static void properties(JSONWriter jsonWriter, CommandMessage commandMessage, MessageLocalisation messageLocalisation) throws IOException {
        jsonWriter.key("type");
        if (commandMessage.isErrorMessage()) {
            jsonWriter.value("error");
        } else {
            jsonWriter.value("done");
        }
        jsonWriter.key("key");
        jsonWriter.value(commandMessage.getMessageKey());
        jsonWriter.key("values");
        jsonWriter.array();
        for (Object value : commandMessage.getMessageValues()) {
            jsonWriter.value(value);
        }
        jsonWriter.endArray();
        if (messageLocalisation != null) {
            jsonWriter.key("text");
            jsonWriter.value(messageLocalisation.toString(commandMessage));
        }
        if (commandMessage.hasLog()) {
            jsonWriter.key("log");
            LogUtils.writeJsonValue(jsonWriter, commandMessage.getMessageLog());
        }
        if (commandMessage.hasMultiError()) {
            jsonWriter.key("multiErrorMessageArray");
            jsonWriter.array();
            for (Message multiErrorMessage : commandMessage.getMultiErrorList()) {
                jsonWriter.object();
                jsonWriter.key("key");
                jsonWriter.value(multiErrorMessage.getMessageKey());
                jsonWriter.key("values");
                jsonWriter.array();
                for (Object value : commandMessage.getMessageValues()) {
                    jsonWriter.value(value);
                }
                jsonWriter.endArray();
                if (messageLocalisation != null) {
                    jsonWriter.key("text");
                    jsonWriter.value(messageLocalisation.toString(multiErrorMessage));
                }
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
        }
    }

    public static void object(JSONWriter jsonWriter, Message message, MessageLocalisation messageLocalisation) throws IOException {
        jsonWriter.object();
        properties(jsonWriter, message, messageLocalisation);
        jsonWriter.endObject();
    }

    public static void properties(JSONWriter jsonWriter, Message message, MessageLocalisation messageLocalisation) throws IOException {
        if (message instanceof CommandMessage) {
            properties(jsonWriter, (CommandMessage) message, messageLocalisation);
            return;
        }
        jsonWriter.key("key");
        jsonWriter.value(message.getMessageKey());
        jsonWriter.key("text");
        jsonWriter.value(messageLocalisation.toString(message));
        Object[] values = message.getMessageValues();
        if (values.length > 0) {
            jsonWriter.key("valueArray");
            jsonWriter.array();
            for (Object value : values) {
                jsonWriter.value(value);
            }
            jsonWriter.endArray();
        }
    }

    public static void properties(JSONWriter jsonWriter, MessageByLine messageByLine, MessageLocalisation messageLocalisation) throws IOException {
        jsonWriter.key("line");
        jsonWriter.value(messageByLine.getLineNumber());
        jsonWriter.key("categoryArray");
        jsonWriter.array();
        for (MessageByLine.Category category : messageByLine.getCategoryList()) {
            jsonWriter.object();
            jsonWriter.key("name");
            jsonWriter.value(category.getName());
            jsonWriter.key("messageArray");
            jsonWriter.array();
            for (Message message : category.getMessageList()) {
                jsonWriter.object();
                properties(jsonWriter, message, messageLocalisation);
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

    public static void title(JSONWriter jsonWriter, LabelHolder labelHolder, Lang lang) throws IOException {
        jsonWriter.key("title");
        Label label = LabelUtils.getLangPartCheckedLabel(labelHolder, lang);
        if (label != null) {
            jsonWriter.value(label.getLabelString());
        } else {
            jsonWriter.value("");
        }
    }

    public static void defaultLabel(JSONWriter jsonWriter, LabelHolder labelHolder, Lang lang, String key, String defaultString) throws IOException {
        Label label = LabelUtils.getLangPartCheckedLabel(labelHolder, lang);
        if (label == null) {
            label = LabelUtils.getFirstLabel(labelHolder);
        }
        String value = defaultString;
        if (label != null) {
            value = label.getLabelString();
        }
        if (value != null) {
            jsonWriter.key(key);
            jsonWriter.value(value);
        }
    }

}
